/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.client.configuration;

import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.ow2.petals.registry_overlay.client.configuration.generated.Configuration;
import org.ow2.petals.registry_overlay.client.configuration.generated.ObjectFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * The builder allows to unmarshall the configuration parameters of the petals
 * registry overlay client located in the topology definition.
 * 
 * @author Christophe DENEUX - Linagora
 */
public final class ConfigurationBuilder {

    /**
     * XML constants about the configuration. <b>MUST</b> be synchronized with
     * the XSD.
     * 
     * @author Christophe DENEUX - Linagora
     */
    public static class XML {

        /**
         * Namespace of the configuration
         */
        public static final String CONFIGURATION_NAMESPACE = "http://petals.ow2.org/petals-registry-overlay-client/configuration";

        /**
         * Name of the root element of the configuration
         */
        public static final String CONFIGURATION_ROOT_ELEMENT_NAME = "configuration";

        /**
         * Name of the group name element of the configuration
         */
        public static final String CONFIGURATION_GROUPNAME_ELEMENT_NAME = "group-name";

        /**
         * Name of the group password element of the configuration
         */
        public static final String CONFIGURATION_GROUPPWD_ELEMENT_NAME = "group-password";

        /**
         * Name of the overlay members element of the configuration
         */
        public static final String CONFIGURATION_OVERLAYMEMBERS_ELEMENT_NAME = "overlay-members";

        /**
         * Name of the overlay member element of the configuration
         */
        public static final String CONFIGURATION_OVERLAYMEMBER_ELEMENT_NAME = "overlay-member";

        /**
         * Name of the overlay member port attribute of the configuration
         */
        public static final String CONFIGURATION_OVERLAYMEMBERPORT_ATTR_NAME = "port";

    }

    /**
     * XSD schema
     */
    private static final String NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";

    /**
     * Topology schema
     */
    private static final String CONFIGURATION_XSD = "petalsRegistryOverlayCltCfg.xsd";

    /**
     * The exception raised during JAXB context creation
     */
    private static ConfigurationException jaxbContextCreationEx;

    /**
     * The unmarshaller.
     */
    private static Unmarshaller unmarshaller;

    /**
     * The exception raised during JAXB unique unmarshaller creation.
     */
    private static ConfigurationException unmarshCreationEx;

    /**
     * The marshaller.
     */
    private static Marshaller marshaller;

    /**
     * The exception raised during JAXB unique marshaller creation.
     */
    private static ConfigurationException marshCreationEx;

    /**
     * Static object initialization
     */
    static {

        final InputStream schemaInputStream = ConfigurationBuilder.class.getResourceAsStream("/"
                + ConfigurationBuilder.CONFIGURATION_XSD);

        try {
            final SchemaFactory factory = SchemaFactory.newInstance(ConfigurationBuilder.NAMESPACE_XSD);
            final Source[] schemas = new StreamSource[] { new StreamSource(schemaInputStream) };
            final Schema schema = factory.newSchema(schemas);

            try {
                final JAXBContext jaxbContext = JAXBContext
                        .newInstance(new Class[] { Configuration.class });

                try {

                    ConfigurationBuilder.unmarshaller = jaxbContext.createUnmarshaller();
                    ConfigurationBuilder.unmarshaller.setSchema(schema);

                } catch (final JAXBException ex) {
                    ConfigurationBuilder.unmarshCreationEx = new ConfigurationException(
                            "Failed to create the JAXB unmarshaller", ex);
                }

                try {

                    ConfigurationBuilder.marshaller = jaxbContext.createMarshaller();
                    ConfigurationBuilder.marshaller.setSchema(schema);
                    ConfigurationBuilder.marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
                    ConfigurationBuilder.marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
                            Boolean.TRUE);

                } catch (final JAXBException ex) {
                    ConfigurationBuilder.marshCreationEx = new ConfigurationException(
                            "Failed to create the JAXB marshaller", ex);
                }
            } catch (JAXBException ex) {
                ConfigurationBuilder.jaxbContextCreationEx = new ConfigurationException(
                        "Failed to create the JAXB context", ex);
            }

        } catch (final SAXException e) {
            final ConfigurationException schemaCreationEx = new ConfigurationException(
                    "Failed to parse resource '" + ConfigurationBuilder.CONFIGURATION_XSD + "'", e);
            ConfigurationBuilder.jaxbContextCreationEx = new ConfigurationException(
                    "Failed to create the JAXB context", schemaCreationEx);
            ConfigurationBuilder.unmarshCreationEx = new ConfigurationException(
                    "Failed to create the JAXB unmarshaller", schemaCreationEx);
        }
    }

    /**
     * No constructor available.
     */
    private ConfigurationBuilder() {
        // NOP
    }

    /**
     * Create the configuration object.
     * 
     * @param xmlNode
     *            A xml node (coming from an any element zone) to parse.
     * @return The configuration, or <code>null</code> if the {@link Node} is
     *         not an expected node.
     * @throws ConfigurationException
     *             impossible to create the topology
     */
    public static Configuration createConfiguration(final Node xmlNode)
            throws ConfigurationException {

        if (ConfigurationBuilder.unmarshaller == null) {
            throw ConfigurationBuilder.unmarshCreationEx;
        }
        
        try {
            final JAXBElement<Configuration> root;
            // The default Xerces unmarshaller is not thread safe
            synchronized (ConfigurationBuilder.unmarshaller) {
                root = ConfigurationBuilder.unmarshaller.unmarshal(xmlNode,
                        Configuration.class);
            }

            return root.getValue();
        } catch (final UnmarshalException e) {
            if (e.getLinkedException().getMessage().contains("cvc-elt.1")) {
                // The declaration of the node element cannot be found. It's not
                // an elment associated to the confoguration of Petals Registry
                // Overlay client.
                return null;
            } else {
                throw new ConfigurationException("Failed to build the configuration", e);
            }
        } catch (final JAXBException e) {
            throw new ConfigurationException("Failed to build the configuration", e);
        }
    }

    /**
     * Write the configuration of the Petals Registry Overlay Client into an DOM {@link Document}
     * 
     * @param petalsRegistryOverlayCfg
     *            The configuration of the Petals Registry Overlay Client
     * @param document
     *            The target DOM {@link Document} in which the configuration will be marshaled
     */
    public synchronized static void marshallConfiguration(final Configuration configuration, final Document document)
            throws ConfigurationException {

        if (ConfigurationBuilder.marshaller == null) {
            throw ConfigurationBuilder.marshCreationEx;
        }

        try {
            final ObjectFactory jaxbObjectFactory = new ObjectFactory();

            final JAXBElement<Configuration> configurationElement = jaxbObjectFactory
                    .createConfiguration(configuration);

            final DOMResult domResult = new DOMResult(document);
            ConfigurationBuilder.marshaller.marshal(configurationElement, domResult);

        } catch (JAXBException ex) {
            throw new ConfigurationException("Failed to write the configuration of the Petals Registry Overlay Client",
                    ex);
        }
    }
}
