/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.client.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.net.URISyntaxException;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.registry_overlay.client.configuration.generated.Configuration;
import org.ow2.petals.topology.TopologyBuilder;
import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.Topology;
import org.w3c.dom.Node;

/**
 * Unit tests of {@link ConfigurationBuilder}
 * 
 * @author Christophe DENEUX - Linagora
 */
public class ConfigurationBuilderTest {

    /**
     * <p>
     * Unit test of the {@link ConfigurationBuilder} about the parsing of the
     * Petals Registry Overlay client configuration located in a topology
     * definition file:
     * <ol>
     * <li>only the configuration of the registry overlay client is available as
     * extra parameters</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error</li>
     * <li>the configuration is correctly retrieved.</li>
     * </ol>
     * </p>
     * 
     */
    @Test
    public void testCreateConfiguration_00() throws ConfigurationException, URISyntaxException,
            TopologyException {
        final String fileName = new File(Thread.currentThread().getContextClassLoader()
                .getResource("topology.xml").toURI()).getAbsolutePath();
        final Topology topology = TopologyBuilder.createTopology(fileName);
        assertNotNull(topology);
        final Domain domain = topology.getDomain();
        assertNotNull(domain);
        final List<Object> extraParameters = domain.getAny();
        assertNotNull(extraParameters);
        assertEquals(1, extraParameters.size());
        final Configuration cfg = ConfigurationBuilder.createConfiguration((Node) extraParameters
                .get(0));
        assertNotNull(cfg);
        assertNotNull(cfg.getOverlayMembers());
        assertNotNull(cfg.getOverlayMembers().getOverlayMember());
        assertEquals(2, cfg.getOverlayMembers().getOverlayMember().size());
    }

    /**
     * <p>
     * Unit test of the {@link ConfigurationBuilder} about the parsing of the
     * Petals Registry Overlay client configuration located in a topology
     * definition file:
     * <ol>
     * <li>the configuration of the registry overlay client is available as
     * extra parameters,</li>
     * <li>another extra parameter is available.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error</li>
     * <li>the configuration is correctly retrieved,</li>
     * <li>{@link ConfigurationBuilder#createConfiguration(Node)} return
     * <code>null</code> on the unknown extra parameter.</li>
     * </ol>
     * </p>
     * 
     */
    @Test
    public void testCreateConfiguration_01() throws ConfigurationException, URISyntaxException,
            TopologyException {
        final String fileName = new File(Thread.currentThread().getContextClassLoader()
                .getResource("topology-with-other-extra-params.xml").toURI()).getAbsolutePath();
        final Topology topology = TopologyBuilder.createTopology(fileName);
        assertNotNull(topology);
        final Domain domain = topology.getDomain();
        assertNotNull(domain);
        final List<Object> extraParameters = domain.getAny();
        assertNotNull(extraParameters);
        assertEquals(2, extraParameters.size());
        boolean otherParamFound = false;
        boolean configFound = false;
        for (final Object extraParameter : extraParameters) {
            final Configuration cfg = ConfigurationBuilder
                    .createConfiguration((Node) extraParameter);
            if (cfg == null) {
                otherParamFound = true;
            } else {
                configFound = true;
            }
        }
        assertTrue(configFound);
        assertTrue(otherParamFound);

    }

    /**
     * <p>
     * Unit test of the {@link ConfigurationBuilder} about the parsing of the
     * Petals Registry Overlay client configuration located in a topology
     * definition file:
     * <ol>
     * <li>the configuration of the registry overlay client is available as
     * extra parameters,</li>
     * <li>the configuration of the registry overlay client is invalid.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>an error is thrown.</li>
     * </ol>
     * </p>
     */
    @Test(expected = ConfigurationException.class)
    public void testCreateConfiguration_02() throws ConfigurationException, URISyntaxException,
            TopologyException {
        final String fileName = new File(Thread.currentThread().getContextClassLoader()
                .getResource("topology-with-error-in-config.xml").toURI()).getAbsolutePath();
        final Topology topology = TopologyBuilder.createTopology(fileName);
        assertNotNull(topology);
        final Domain domain = topology.getDomain();
        assertNotNull(domain);
        final List<Object> extraParameters = domain.getAny();
        assertNotNull(extraParameters);
        assertEquals(1, extraParameters.size());
        ConfigurationBuilder.createConfiguration((Node) extraParameters
                .get(0));

    }
}